@@ -44,6 +44,7 @@
 #include "rb-encoder.h"
 #include "rb-encoder-gst.h"
 #include "rb-debug.h"
+#include "rb-util.h"
 
 #ifdef HAVE_GSTREAMER_0_8
 #define GstStateChangeReturn GstElementStateReturn
@@ -67,7 +68,8 @@ struct _RBEncoderGstPrivate {
 	gboolean error_emitted;
 	gboolean completion_emitted;
 
-	gulong total_length;
+	GstFormat position_format;
+	gint64 total_length;
 	guint progress_id;
 };
 
@@ -79,18 +81,45 @@ G_DEFINE_TYPE_WITH_CODE(RBEncoderGst, rb_encoder_gst, G_TYPE_OBJECT,
 static gboolean rb_encoder_gst_encode (RBEncoder *encoder,
 				       RhythmDBEntry *entry,
 				       const char *dest,
-				       const char *mime_type);
+				       GList *mime_types);
 static void rb_encoder_gst_cancel (RBEncoder *encoder);
+static gboolean rb_encoder_gst_get_preferred_mimetype (RBEncoder *encoder,
+						       GList *mime_types,
+						       char **mime,
+						       char **extension);
 static void rb_encoder_gst_emit_completed (RBEncoderGst *encoder);
 
+
 static void
 rb_encoder_gst_class_init (RBEncoderGstClass *klass)
 {
         GObjectClass *object_class = (GObjectClass *) klass;
+	GstCaps *caps;
 
         object_class->finalize = rb_encoder_gst_finalize;
 
         g_type_class_add_private (klass, sizeof (RBEncoderGstPrivate));
+
+	/* create the mimetype -> GstCaps lookup table
+	 *
+	 * The strings are static data for now, but if we allow dynamic changing
+	 * we need to change this to use g_strdup/g_free
+	 */
+	klass->mime_caps_table = g_hash_table_new_full (g_str_hash, g_str_equal,
+							NULL, (GDestroyNotify)gst_caps_unref);
+
+	/* AAC */
+	caps = gst_caps_new_simple ("audio/mpeg",
+				    "mpegversion", G_TYPE_INT, 4,
+				    NULL);
+	g_hash_table_insert (klass->mime_caps_table, "audio/aac", caps);
+
+	/* MP3 */
+	caps = gst_caps_new_simple ("audio/mpeg",
+				    "mpegversion", G_TYPE_INT, 1,
+				    "layer", G_TYPE_INT, 3,
+				    NULL);
+	g_hash_table_insert (klass->mime_caps_table, "audio/mpeg", caps);
 }
 
 static void
@@ -104,6 +133,7 @@ rb_encoder_init (RBEncoderIface *iface)
 {
 	iface->encode = rb_encoder_gst_encode;
 	iface->cancel = rb_encoder_gst_cancel;
+	iface->get_preferred_mimetype = rb_encoder_gst_get_preferred_mimetype;
 }
 
 static void
@@ -247,20 +277,21 @@ gst_error_cb (GstElement *element,
 static gboolean
 progress_timeout_cb (RBEncoderGst *encoder)
 {
-	gint64 nanos;
-	gint secs;
-	static GstFormat format = GST_FORMAT_TIME;
+	gint64 position;
+	static GstFormat format;
 	GstState state;
 
 	if (encoder->priv->pipeline == NULL)
 		return FALSE;
 
+	format = encoder->priv->position_format;
+
 #ifdef HAVE_GSTREAMER_0_10
 	gst_element_get_state (encoder->priv->pipeline, &state, NULL, GST_CLOCK_TIME_NONE);
 	if (state != GST_STATE_PLAYING)
 		return FALSE;
 
-	if (!gst_element_query_position (encoder->priv->pipeline, &format, &nanos)) {
+	if (!gst_element_query_position (encoder->priv->pipeline, &format, &position)) {
 		g_warning ("Could not get current track position");
 		return TRUE;
 	}
@@ -269,15 +300,28 @@ progress_timeout_cb (RBEncoderGst *encoder)
 	if (state != GST_STATE_PLAYING)
 		return FALSE;
 
-	if (!gst_element_query (encoder->priv->pipeline, GST_QUERY_POSITION, &format, &nanos)) {
+	if (!gst_element_query (encoder->priv->pipeline, GST_QUERY_POSITION, &format, &position)) {
 		g_warning ("Could not get current track position");
 		return TRUE;
 	}
 #endif
 
-	secs = nanos / GST_SECOND;
-	rb_debug ("encoding progress at %d out of %lu", secs, encoder->priv->total_length);
-	_rb_encoder_emit_progress (RB_ENCODER (encoder), ((double)secs) / encoder->priv->total_length);
+	if (format == GST_FORMAT_TIME) {
+		gint secs;
+
+		secs = position / GST_SECOND;
+		rb_debug ("encoding progress at %d out of %" G_GINT64_FORMAT,
+			  secs,
+			  encoder->priv->total_length);
+		_rb_encoder_emit_progress (RB_ENCODER (encoder),
+					   ((double)secs) / encoder->priv->total_length);
+	} else {
+		rb_debug ("encoding progress at %" G_GINT64_FORMAT " out of %" G_GINT64_FORMAT,
+			  position,
+			  encoder->priv->total_length);
+		_rb_encoder_emit_progress (RB_ENCODER (encoder),
+					   ((double) position) / encoder->priv->total_length);
+	}
 
 	return TRUE;
 }
@@ -602,21 +646,59 @@ attach_output_pipeline (RBEncoderGst *encoder,
 }
 
 static gboolean
-encoder_match_mime (GstElement *encoder, const gchar *mime_type)
+encoder_match_mime (RBEncoderGst *rbencoder, GstElement *encoder, const gchar *mime_type)
 {
 	GstPad *srcpad;
-	GstCaps *caps;
-	GstStructure *structure;
-	const gchar *pad_mime;
-	gboolean match;
+	GstCaps *element_caps = NULL;
+	GstCaps *desired_caps = NULL;
+	GstCaps *intersect_caps = NULL;
+	gboolean match = FALSE;
+	char *tmp;
 
 	srcpad = gst_element_get_pad (encoder, "src");
-	caps = gst_pad_get_caps (srcpad);
-	structure = gst_caps_get_structure (caps, 0);
-	pad_mime = gst_structure_get_name (structure);
-	match = strcmp (mime_type, pad_mime) == 0;
-	gst_caps_unref (caps);
-	gst_object_unref (GST_OBJECT (srcpad));
+	element_caps = gst_pad_get_caps (srcpad);
+
+	if (element_caps == NULL) {
+		g_warning ("couldn't create any element caps");
+		goto end;
+	}
+
+	desired_caps = g_hash_table_lookup (RB_ENCODER_GST_GET_CLASS (rbencoder)->mime_caps_table, mime_type);
+	if (desired_caps != NULL) {
+		gst_caps_ref (desired_caps);
+	} else {
+		desired_caps = gst_caps_new_simple (mime_type, NULL);
+	}
+
+	if (desired_caps == NULL) {
+		g_warning ("couldn't create any desired caps for mimetype: %s", mime_type);
+		goto end;
+	}
+
+	intersect_caps = gst_caps_intersect (desired_caps, element_caps);
+	match = !gst_caps_is_empty (intersect_caps);
+
+	tmp = gst_caps_to_string (desired_caps);
+	rb_debug ("desired caps are: %s", tmp);
+	g_free (tmp);
+
+	tmp = gst_caps_to_string (element_caps);
+	rb_debug ("element caps are: %s", tmp);
+	g_free (tmp);
+
+	tmp = gst_caps_to_string (intersect_caps);
+	rb_debug ("intersect caps are: %s", tmp);
+	g_free (tmp);
+
+end:
+	if (intersect_caps != NULL)
+		gst_caps_unref (intersect_caps);
+	if (desired_caps != NULL)
+		gst_caps_unref (desired_caps);
+	if (element_caps != NULL)
+		gst_caps_unref (element_caps);
+	if (srcpad != NULL)
+		gst_object_unref (GST_OBJECT (srcpad));
 
 	return match;
 }
@@ -637,7 +719,7 @@ profile_bin_find_encoder (GstBin *profile_bin)
 		switch (gst_iterator_next (iter, &data)) {
 			case GST_ITERATOR_OK:
 				factory = gst_element_get_factory (GST_ELEMENT (data));
-				if (strcmp (factory->details.klass,
+				if (rb_safe_strcmp (factory->details.klass,
 						"Codec/Encoder/Audio") == 0) {
 					encoder = GST_ELEMENT (data);
 					done = TRUE;
@@ -665,7 +747,7 @@ profile_bin_find_encoder (GstBin *profile_bin)
 }
 
 static GMAudioProfile*
-get_profile_from_mime_type (const char *mime_type)
+get_profile_from_mime_type (RBEncoderGst *rbencoder, const char *mime_type)
 {
 	GList *profiles, *walk;
 	gchar *pipeline_description;
@@ -675,6 +757,8 @@ get_profile_from_mime_type (const char *mime_type)
 	GMAudioProfile *matching_profile = NULL;
 	GError *error = NULL;
 
+	rb_debug ("Looking up profile for mimetype '%s'", mime_type);
+
 	profiles = gm_audio_profile_get_active_list ();
 	for (walk = profiles; walk; walk = g_list_next (walk)) {
 		profile = (GMAudioProfile *) walk->data;
@@ -695,7 +779,7 @@ get_profile_from_mime_type (const char *mime_type)
 			continue;
 		}
 
-		if (encoder_match_mime (encoder, mime_type)) {
+		if (encoder_match_mime (rbencoder, encoder, mime_type)) {
 			matching_profile = profile;
 			gst_object_unref (GST_OBJECT (encoder));
 			gst_object_unref (GST_OBJECT (pipeline));
@@ -706,9 +790,29 @@ get_profile_from_mime_type (const char *mime_type)
 		gst_object_unref (GST_OBJECT (pipeline));
 	}
 
+	if (matching_profile)
+		g_object_ref (matching_profile);
+	g_list_free (profiles);
+
 	return matching_profile;
 }
 
+static GMAudioProfile*
+get_profile_from_mime_types (RBEncoderGst *rbencoder, GList *mime_types)
+{
+	GList *l;
+
+	for (l = mime_types; l != NULL; l = g_list_next (l)) {
+		GMAudioProfile *profile;
+
+		profile = get_profile_from_mime_type (rbencoder, (const char *)l->data);
+		if (profile)
+			return profile;
+	}
+
+	return NULL;
+}
+
 static GstElement *
 create_pipeline_and_source (RBEncoderGst *encoder,
 			    RhythmDBEntry *entry,
@@ -830,7 +934,7 @@ static gboolean
 transcode_track (RBEncoderGst *encoder,
 	 	 RhythmDBEntry *entry,
 		 const char *dest,
-		 const char *mime_type,
+		 GList *mime_types,
 		 GError **error)
 {
 	/* src ! decodebin ! queue ! encoding_profile ! queue ! sink */
@@ -839,14 +943,14 @@ transcode_track (RBEncoderGst *encoder,
 
 	g_assert (encoder->priv->pipeline == NULL);
 
-	profile = get_profile_from_mime_type (mime_type);
+	profile = get_profile_from_mime_types (encoder, mime_types);
 	if (profile == NULL) {
 		g_set_error (error,
 			     RB_ENCODER_ERROR,
 			     RB_ENCODER_ERROR_FORMAT_UNSUPPORTED,
 			     "Unable to locate encoding profile for mime-type "
-			     "'%s'", mime_type);
-		return FALSE;
+			     /*"'%s'", mime_type*/);
+		goto error;
 	} else {
 		rb_debug ("selected profile %s",
 				gm_audio_profile_get_name (profile));
@@ -854,27 +958,32 @@ transcode_track (RBEncoderGst *encoder,
 
 	src = create_pipeline_and_source (encoder, entry, error);
 	if (src == NULL)
-		return FALSE;
+		goto error;
 
 	decoder = add_decoding_pipeline (encoder, error);
 	if (decoder == NULL)
-		return FALSE;
+		goto error;
 
 	if (gst_element_link (src, decoder) == FALSE)
-		return FALSE;
+		goto error;
 
 	end = add_encoding_pipeline (encoder, profile, error);
 	if (end == NULL)
-		return FALSE;
+		goto error;
 
 	if (!attach_output_pipeline (encoder, end, dest, error))
-		return FALSE;
+		goto error;
 	if (!add_tags_from_entry (encoder, entry, error))
-		return FALSE;
+		goto error;
 	if (!start_pipeline (encoder, error))
-		return FALSE;
+		goto error;
 
 	return TRUE;
+error:
+	if (profile)
+		g_object_unref (profile);
+
+	return FALSE;
 }
 
 static GnomeVFSResult
@@ -931,10 +1040,11 @@ static gboolean
 rb_encoder_gst_encode (RBEncoder *encoder,
 		       RhythmDBEntry *entry,
 		       const char *dest,
-		       const char *mime_type)
+		       GList *mime_types)
 {
 	RBEncoderGstPrivate *priv = RB_ENCODER_GST (encoder)->priv;
 	const char *entry_mime_type;
+	gboolean copy;
 	gboolean was_raw;
 	gboolean result;
 	GError *error = NULL;
@@ -945,7 +1055,16 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 	entry_mime_type = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE);
 	was_raw = g_str_has_prefix (entry_mime_type, "audio/x-raw");
 
-	priv->total_length = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
+	/* hackish mapping of gstreamer media types to mime types; this
+	 * should be easier when we do proper (deep) typefinding.
+	 */
+	if (rb_safe_strcmp (entry_mime_type, "audio/x-wav") == 0) {
+		/* if it has a bitrate, assume it's mp3-in-wav */
+		if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE) != 0)
+			entry_mime_type = "audio/mpeg";
+	} else if (rb_safe_strcmp (entry_mime_type, "application/x-id3") == 0) {
+		entry_mime_type = "audio/mpeg";
+	}
 
 	vfsresult = create_parent_dirs (dest);
 	if (vfsresult != GNOME_VFS_OK) {
@@ -959,13 +1078,38 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 		return FALSE;
 	}
 
-	if ((mime_type == NULL && !was_raw) || (mime_type && (strcmp (mime_type, entry_mime_type) == 0))) {
+	if (mime_types == NULL) {
+		/* don't copy raw audio */
+		copy = !was_raw;
+	} else {
+		GList *l;
+
+		/* see if it's already in any of the destination formats */
+		copy = FALSE;
+		for (l = mime_types; l != NULL; l = g_list_next (l)) {
+			rb_debug ("Comparing mimetypes '%s' '%s'", entry_mime_type, (char *)l->data);
+			if (rb_safe_strcmp (entry_mime_type, l->data) == 0) {
+				rb_debug ("Matched mimetypes '%s' '%s'", entry_mime_type, (char *)l->data);
+
+				copy = TRUE;
+				break;
+			}
+		}
+	}
+
+	if (copy) {
+		priv->total_length = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
+		priv->position_format = GST_FORMAT_BYTES;
+
 		result = copy_track (RB_ENCODER_GST (encoder), entry, dest, &error);
 	} else {
-		if (mime_type == NULL) {
+		priv->total_length = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
+		priv->position_format = GST_FORMAT_TIME;
+
+		if (mime_types == NULL) {
 			result = extract_track (RB_ENCODER_GST (encoder), entry, dest, &error);
 		} else {
-			result = transcode_track (RB_ENCODER_GST (encoder), entry, dest, mime_type, &error);
+			result = transcode_track (RB_ENCODER_GST (encoder), entry, dest, mime_types, &error);
 		}
 	}
 
@@ -984,3 +1128,32 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 
 	return result;
 }
+
+static gboolean
+rb_encoder_gst_get_preferred_mimetype (RBEncoder *encoder,
+				       GList *mime_types,
+				       char **mime,
+				       char **extension)
+{
+	GList *l;
+
+	g_return_val_if_fail (mime_types != NULL, FALSE);
+	g_return_val_if_fail (mime != NULL, FALSE);
+	g_return_val_if_fail (extension != NULL, FALSE);
+
+	for (l = mime_types; l != NULL; l = g_list_next (l)) {
+		GMAudioProfile *profile;
+		const char *mimetype;
+
+		mimetype = (const char *)l->data;
+		profile = get_profile_from_mime_type (RB_ENCODER_GST (encoder), mimetype);
+		if (profile) {
+			*extension = g_strdup (gm_audio_profile_get_extension (profile));
+			*mime = g_strdup (mimetype);
+			g_object_unref (profile);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
