@@ -54,7 +54,23 @@ extern void rb_marshal_VOID__BOOLEAN_BOOLEAN (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:7) */
+/* BOXED:BOXED (./rb-marshal.list:7) */
+extern void rb_marshal_BOXED__BOXED (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data);
+
+/* VOID:BOXED,BOXED (./rb-marshal.list:8) */
+extern void rb_marshal_VOID__BOXED_BOXED (GClosure     *closure,
+                                          GValue       *return_value,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint,
+                                          gpointer      marshal_data);
+
+/* VOID:BOXED,INT,POINTER,POINTER (./rb-marshal.list:9) */
 extern void rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -62,7 +78,7 @@ extern void rb_marshal_VOID__BOXED_INT_POINTER_POINTER (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:BOXED,POINTER (./rb-marshal.list:8) */
+/* VOID:BOXED,POINTER (./rb-marshal.list:10) */
 extern void rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -70,7 +86,15 @@ extern void rb_marshal_VOID__BOXED_POINTER (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:BOXED,ULONG (./rb-marshal.list:9) */
+/* VOID:BOXED,STRING,BOXED (./rb-marshal.list:11) */
+extern void rb_marshal_VOID__BOXED_STRING_BOXED (GClosure     *closure,
+                                                 GValue       *return_value,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint,
+                                                 gpointer      marshal_data);
+
+/* VOID:BOXED,ULONG (./rb-marshal.list:12) */
 extern void rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -78,7 +102,7 @@ extern void rb_marshal_VOID__BOXED_ULONG (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:DOUBLE,LONG (./rb-marshal.list:10) */
+/* VOID:DOUBLE,LONG (./rb-marshal.list:13) */
 extern void rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -86,7 +110,7 @@ extern void rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:INT,INT,DOUBLE (./rb-marshal.list:11) */
+/* VOID:INT,INT,DOUBLE (./rb-marshal.list:14) */
 extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -94,7 +118,7 @@ extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:INT,POINTER (./rb-marshal.list:12) */
+/* VOID:INT,POINTER (./rb-marshal.list:15) */
 extern void rb_marshal_VOID__INT_POINTER (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -102,7 +126,7 @@ extern void rb_marshal_VOID__INT_POINTER (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT (./rb-marshal.list:13) */
+/* VOID:OBJECT,INT,INT (./rb-marshal.list:16) */
 extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -110,7 +134,7 @@ extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:14) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:17) */
 extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -118,7 +142,7 @@ extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closu
                                                              gpointer      invocation_hint,
                                                              gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:15) */
+/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:18) */
 extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -126,7 +150,7 @@ extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:POINTER,INT (./rb-marshal.list:16) */
+/* VOID:POINTER,INT (./rb-marshal.list:19) */
 extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -134,7 +158,7 @@ extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:POINTER,INT,POINTER (./rb-marshal.list:17) */
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:20) */
 extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -142,7 +166,7 @@ extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER (./rb-marshal.list:18) */
+/* VOID:POINTER,POINTER (./rb-marshal.list:21) */
 extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -150,7 +174,7 @@ extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:POINTER,ULONG (./rb-marshal.list:19) */
+/* VOID:POINTER,ULONG (./rb-marshal.list:22) */
 extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -158,7 +182,7 @@ extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,DOUBLE (./rb-marshal.list:20) */
+/* VOID:STRING,DOUBLE (./rb-marshal.list:23) */
 extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -166,7 +190,7 @@ extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,INT (./rb-marshal.list:21) */
+/* VOID:STRING,INT (./rb-marshal.list:24) */
 extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -174,7 +198,7 @@ extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:STRING,POINTER (./rb-marshal.list:22) */
+/* VOID:STRING,POINTER (./rb-marshal.list:25) */
 extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -182,7 +206,7 @@ extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING,STRING (./rb-marshal.list:23) */
+/* VOID:STRING,STRING (./rb-marshal.list:26) */
 extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -190,7 +214,7 @@ extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING (./rb-marshal.list:24) */
+/* VOID:STRING,STRING,STRING (./rb-marshal.list:27) */
 extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -198,7 +222,7 @@ extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:25) */
+/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:28) */
 extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -206,7 +230,7 @@ extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *cl
                                                                 gpointer      invocation_hint,
                                                                 gpointer      marshal_data);
 
-/* VOID:ULONG,FLOAT (./rb-marshal.list:26) */
+/* VOID:ULONG,FLOAT (./rb-marshal.list:29) */
 extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -214,7 +238,7 @@ extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:27) */
+/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:30) */
 extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -222,7 +246,7 @@ extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:28) */
+/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:31) */
 extern void rb_marshal_VOID__STRING_STRING_POINTER_POINTER (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
