@@ -25,23 +25,11 @@
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-
 #include <libgnomevfs/gnome-vfs.h>
 
-#include <nautilus-burn-drive.h>
-#ifndef NAUTILUS_BURN_CHECK_VERSION 	 
-#define NAUTILUS_BURN_CHECK_VERSION(a,b,c) FALSE 	 
-#endif
-
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-#include <nautilus-burn.h>
-#endif
-
 #include "rb-removable-media-manager.h"
 #include "rb-library-source.h"
-#include "rb-sourcelist.h"
 #include "rb-removable-media-source.h"
-#include "rb-audiocd-source.h"
 
 #include "rb-shell.h"
 #include "rb-shell-player.h"
@@ -56,10 +44,6 @@
 #include "rb-encoder.h"
 #endif
 
-#ifndef HAVE_BURN_DRIVE_UNREF
-#define nautilus_burn_drive_unref nautilus_burn_drive_free
-#endif
-
 static void rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass);
 static void rb_removable_media_manager_init (RBRemovableMediaManager *mgr);
 static void rb_removable_media_manager_dispose (GObject *object);
@@ -77,7 +61,7 @@ static void rb_removable_media_manager_cmd_scan_media (GtkAction *action,
 						       RBRemovableMediaManager *manager);
 static void rb_removable_media_manager_cmd_eject_medium (GtkAction *action,
 					       RBRemovableMediaManager *mgr);
-static void rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr, 
+static void rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 					     GtkUIManager *uimanager);
 
 static void rb_removable_media_manager_append_media_source (RBRemovableMediaManager *mgr, RBRemovableMediaSource *source);
@@ -88,10 +72,10 @@ static void rb_removable_media_manager_unmount_volume (RBRemovableMediaManager *
 				GnomeVFSVolume *volume);
 
 static void  rb_removable_media_manager_volume_mounted_cb (GnomeVFSVolumeMonitor *monitor,
-				GnomeVFSVolume *volume, 
+				GnomeVFSVolume *volume,
 				gpointer data);
 static void  rb_removable_media_manager_volume_unmounted_cb (GnomeVFSVolumeMonitor *monitor,
-				GnomeVFSVolume *volume, 
+				GnomeVFSVolume *volume,
 				gpointer data);
 static gboolean rb_removable_media_manager_load_media (RBRemovableMediaManager *manager);
 
@@ -106,7 +90,6 @@ typedef struct
 	RBShell *shell;
 	gboolean disposed;
 
-	RBSourceList *sourcelist;
 	RBSource *selected_source;
 
 	GtkActionGroup *actiongroup;
@@ -114,14 +97,9 @@ typedef struct
 
 	GList *sources;
 	GHashTable *volume_mapping;
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-	GHashTable *cd_drive_mapping;
-#endif
 	GList *cur_volume_list;
 	gboolean scanned;
 
-	char *playing_uri;
-
 	GAsyncQueue *transfer_queue;
 	gboolean transfer_running;
 	gint transfer_total;
@@ -136,7 +114,6 @@ enum
 {
 	PROP_0,
 	PROP_SHELL,
-	PROP_SOURCELIST,
 	PROP_SOURCE,
 	PROP_SCANNED
 };
@@ -165,7 +142,6 @@ static GtkActionEntry rb_removable_media_manager_actions [] =
 };
 static guint rb_removable_media_manager_n_actions = G_N_ELEMENTS (rb_removable_media_manager_actions);
 
-
 static void
 rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 {
@@ -191,13 +167,6 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 							      RB_TYPE_SHELL,
 							      G_PARAM_READWRITE));
 
-	g_object_class_install_property (object_class,
-					 PROP_SOURCELIST,
-					 g_param_spec_object ("sourcelist",
-							      "RBSourceList",
-							      "RBSourceList",
-							      RB_TYPE_SOURCELIST,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
 					 PROP_SCANNED,
 					 g_param_spec_boolean ("scanned",
@@ -225,7 +194,7 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 			      rb_marshal_VOID__INT_INT_DOUBLE,
 			      G_TYPE_NONE,
 			      3, G_TYPE_INT, G_TYPE_INT, G_TYPE_DOUBLE);
-	
+
 	rb_removable_media_manager_signals[CREATE_SOURCE] =
 		g_signal_new ("create-source",
 			      RB_TYPE_REMOVABLE_MEDIA_MANAGER,
@@ -260,11 +229,11 @@ rb_removable_media_manager_dispose (GObject *object)
 	{
 		GnomeVFSVolumeMonitor *monitor = gnome_vfs_get_volume_monitor ();
 
-		g_signal_handlers_disconnect_by_func (G_OBJECT (monitor), 
-						      G_CALLBACK (rb_removable_media_manager_volume_mounted_cb), 
+		g_signal_handlers_disconnect_by_func (G_OBJECT (monitor),
+						      G_CALLBACK (rb_removable_media_manager_volume_mounted_cb),
 						      mgr);
-		g_signal_handlers_disconnect_by_func (G_OBJECT (monitor), 
-						      G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb), 
+		g_signal_handlers_disconnect_by_func (G_OBJECT (monitor),
+						      G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb),
 						      mgr);
 	}
 
@@ -284,11 +253,8 @@ rb_removable_media_manager_finalize (GObject *object)
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (object);
 
 	g_hash_table_destroy (priv->volume_mapping);
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-	g_hash_table_destroy (priv->cd_drive_mapping);
-#endif
 	g_async_queue_unref (priv->transfer_queue);
-	
+
 	G_OBJECT_CLASS (rb_removable_media_manager_parent_class)->finalize (object);
 }
 
@@ -312,22 +278,19 @@ rb_removable_media_manager_set_property (GObject *object,
 		GtkUIManager *uimanager;
 
 		priv->shell = g_value_get_object (value);
-		g_object_get (G_OBJECT (priv->shell), 
-			      "ui-manager", &uimanager, 
+		g_object_get (priv->shell,
+			      "ui-manager", &uimanager,
 			      NULL);
 		rb_removable_media_manager_set_uimanager (RB_REMOVABLE_MEDIA_MANAGER (object), uimanager);
+		g_object_unref (uimanager);
 		break;
 	}
-	case PROP_SOURCELIST:
-		priv->sourcelist = g_value_get_object (value);
-		break;
-	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-static void 
+static void
 rb_removable_media_manager_get_property (GObject *object,
 			      guint prop_id,
 			      GValue *value,
@@ -343,9 +306,6 @@ rb_removable_media_manager_get_property (GObject *object,
 	case PROP_SHELL:
 		g_value_set_object (value, priv->shell);
 		break;
-	case PROP_SOURCELIST:
-		g_value_set_object (value, priv->sourcelist);
-		break;
 	case PROP_SCANNED:
 		g_value_set_boolean (value, priv->scanned);
 		break;
@@ -356,200 +316,20 @@ rb_removable_media_manager_get_property (GObject *object,
 }
 
 RBRemovableMediaManager *
-rb_removable_media_manager_new (RBShell *shell,
-				 RBSourceList *sourcelist)
+rb_removable_media_manager_new (RBShell *shell)
 {
 	return g_object_new (RB_TYPE_REMOVABLE_MEDIA_MANAGER,
 			     "shell", shell,
-			     "sourcelist", sourcelist,
 			     NULL);
 }
 
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-typedef struct
-{
-	gboolean removed;
-	gboolean tray_opened;
-	RBRemovableMediaManager *manager;
-	NautilusBurnDrive *drive;
-} RbCdDriveInfo;
-
-#ifdef HAVE_BURN_DRIVE_DOOR
-static
-gboolean poll_tray_opened (RbCdDriveInfo *info)
-{
-	GnomeVFSVolumeMonitor *monitor =  gnome_vfs_get_volume_monitor ();
-	gboolean new_status;
-	GnomeVFSVolume *volume;
-
-	if (info->removed) {
-		nautilus_burn_drive_unref (info->drive);
-		g_free (info);
-		return FALSE;
-	}
-
-	new_status = nautilus_burn_drive_door_is_open (info->drive);
-
-	if (new_status != info->tray_opened) {
-		volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, info->drive->device);
-		if (volume) {
-			if (new_status)
-				rb_removable_media_manager_unmount_volume (info->manager, volume);
-			else
-				rb_removable_media_manager_mount_volume (info->manager, volume);
-			gnome_vfs_volume_unref (volume);
-		}
-	}
-	info->tray_opened = new_status;
-
-	return TRUE;
-}
-#endif
-
-static
-void end_cd_drive_monitor (RbCdDriveInfo *info, RBRemovableMediaManager *manager)
-{
-	/* this will be freed when the poll next gets called */
-	info->removed = TRUE;
-}
-
-static
-void begin_cd_drive_monitor (NautilusBurnDrive *drive, RBRemovableMediaManager *manager)
-{
-#ifdef HAVE_BURN_DRIVE_DOOR
-	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
-	RbCdDriveInfo *info = g_new0 (RbCdDriveInfo, 1);
-	GnomeVFSVolumeMonitor *monitor=  gnome_vfs_get_volume_monitor ();
-	GnomeVFSVolume *volume;
-
-	info->drive = drive;
-	info->tray_opened = nautilus_burn_drive_door_is_open (drive);
-	info->manager = manager;
-
-	g_hash_table_insert (priv->cd_drive_mapping, drive, info);
-	g_timeout_add (1000, (GSourceFunc)poll_tray_opened, info);
-
-	volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, drive->device);
-
-	if (volume) {
-		if (!nautilus_burn_drive_door_is_open (drive)) {
-			rb_removable_media_manager_mount_volume (manager, volume);
-		} else {
-			/* it may have got ejected while we weren't monitoring */
-			rb_removable_media_manager_unmount_volume (manager, volume);
-		}
-	}
-#endif
-}
-
-static NautilusBurnDrive *
-get_nautilus_burn_drive_for_path (const char *path)
-{
-#ifdef HAVE_BURN_DRIVE_NEW_FROM_PATH
-	return nautilus_burn_drive_new_from_path (path);
-#else
-	GList *drives, *l;
-	NautilusBurnDrive *path_drive = NULL;
-
-	drives = nautilus_burn_drive_get_list (FALSE, FALSE);
-	for (l = drives; l != NULL; l = g_list_next (l)) {
-		NautilusBurnDrive *drive = (NautilusBurnDrive*)l->data;
-
-		if (path_drive == NULL && strcmp (drive->device, path) == 0) {
-			path_drive = drive;
-		} else {
-			nautilus_burn_drive_unref (drive);
-		}
-	}
-	g_list_free (drives);
-
-	return path_drive;
-#endif
-}
-#endif /* NAUTILUS_BURN < 2.15.3 */
-
-static char *
-split_drive_from_cdda_uri (const char *uri)
-{
-	gchar *copy, *temp, *split;
-	int len;
-
-	if (!g_str_has_prefix (uri, "cdda://"))
-		return NULL;
-
-	len = strlen ("cdda://");
-	
-	copy = g_strdup (uri);
-	split = g_utf8_strrchr (copy + len, -1, ':');
-
-	if (split == NULL) {
-		/* invalid URI, it doesn't contain a ':' */
-		g_free (copy);
-		return NULL;
-	}
-
-	*split = 0;
-	temp = g_strdup (copy + len);
-	g_free (copy);
-
-	return temp;
-}
-
-static void
-rb_removable_media_manager_playing_uri_changed_cb (RBShellPlayer *player,
-						   const char *uri,
-						   RBRemovableMediaManager *manager)
-{
-	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
-	char *old_drive = NULL;
-	char *new_drive = NULL;
-
-	/* extract the drive paths */
-	if (priv->playing_uri)
-		old_drive = split_drive_from_cdda_uri (priv->playing_uri);
-
-	if (uri)
-		new_drive = split_drive_from_cdda_uri (uri);
-
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-	/* if the drive we're playing from has changed, adjust the polling */
-	if (old_drive == NULL || new_drive == NULL || strcmp (old_drive, new_drive) != 0) {
-		if (old_drive) {
-			NautilusBurnDrive *drive;
-
-			rb_debug ("restarting monitoring of drive %s after playing", old_drive);
-			drive = get_nautilus_burn_drive_for_path (old_drive);
-			begin_cd_drive_monitor (drive, manager);
-			nautilus_burn_drive_unref (drive);
-		}
-		
-		if (new_drive) {
-			NautilusBurnDrive *drive;
-
-			rb_debug ("stopping monitoring of drive %s while playing", new_drive);
-			drive = get_nautilus_burn_drive_for_path (new_drive);
-			/* removing it from the hash table makes it stop monitoring */
-			g_hash_table_remove (priv->cd_drive_mapping, drive);
-			nautilus_burn_drive_unref (drive);
-		}
-	}
-#endif
-
-	g_free (priv->playing_uri);
-	priv->playing_uri = (uri) ? g_strdup (uri) : NULL;
-}
-
-	
 static gboolean
 rb_removable_media_manager_load_media (RBRemovableMediaManager *manager)
 {
-	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
 	GnomeVFSVolumeMonitor *monitor = gnome_vfs_get_volume_monitor ();
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-	GList *drives;
-#endif
-	GObject *shell_player;
 
+	GDK_THREADS_ENTER ();
+	
 	/*
 	 * Monitor new (un)mounted file systems to look for new media
 	 *
@@ -558,41 +338,23 @@ rb_removable_media_manager_load_media (RBRemovableMediaManager *manager)
 	 * (e.g. someone pressing the eject button on a cd drive). If we get the
 	 * pre-unmount signal, the corrosponding unmounted signal is ignored
 	 */
-	g_signal_connect (G_OBJECT (monitor), "volume-mounted", 
-			  G_CALLBACK (rb_removable_media_manager_volume_mounted_cb), 
+	g_signal_connect (G_OBJECT (monitor), "volume-mounted",
+			  G_CALLBACK (rb_removable_media_manager_volume_mounted_cb),
 			  manager);
-	g_signal_connect (G_OBJECT (monitor), "volume-pre-unmount", 
-			  G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb), 
+	g_signal_connect (G_OBJECT (monitor), "volume-pre-unmount",
+			  G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb),
 			  manager);
-	g_signal_connect (G_OBJECT (monitor), "volume-unmounted", 
-			  G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb), 
-			  manager);
-
-	/*
-	 * Monitor all cd drives for inserted audio cds
-	 *
-	 * This needs to be done seperately from the above, because non-HAL systems don't
-	 * (currently) report audio cd insertions as mount events.
-	 */
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-	priv->cd_drive_mapping = g_hash_table_new_full (NULL, NULL, NULL, (GDestroyNotify)end_cd_drive_monitor);
-	drives = nautilus_burn_drive_get_list (FALSE, FALSE);
-	g_list_foreach (drives, (GFunc)begin_cd_drive_monitor, manager);
-	g_list_free (drives);
-#endif
-
-	/* monitor the playing song, to disable cd drive polling */
-	shell_player = rb_shell_get_player (priv->shell);
-	g_signal_connect (shell_player, "playing-uri-changed",
-			  G_CALLBACK (rb_removable_media_manager_playing_uri_changed_cb),
+	g_signal_connect (G_OBJECT (monitor), "volume-unmounted",
+			  G_CALLBACK (rb_removable_media_manager_volume_unmounted_cb),
 			  manager);
 
+	GDK_THREADS_LEAVE ();
 	return FALSE;
 }
 
-static void 
+static void
 rb_removable_media_manager_volume_mounted_cb (GnomeVFSVolumeMonitor *monitor,
-			   GnomeVFSVolume *volume, 
+			   GnomeVFSVolume *volume,
 			   gpointer data)
 {
 	RBRemovableMediaManager *mgr = RB_REMOVABLE_MEDIA_MANAGER (data);
@@ -600,9 +362,8 @@ rb_removable_media_manager_volume_mounted_cb (GnomeVFSVolumeMonitor *monitor,
 	rb_removable_media_manager_mount_volume (mgr, volume);
 }
 
-
-static gboolean 
-remove_volume_by_source (GnomeVFSVolume *volume, RBSource *source, 
+static gboolean
+remove_volume_by_source (GnomeVFSVolume *volume, RBSource *source,
 			 RBSource *ref_source)
 {
 	return (ref_source == source);
@@ -614,7 +375,7 @@ rb_removable_media_manager_source_deleted_cb (RBSource *source, RBRemovableMedia
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
 
 	rb_debug ("removing source %p", source);
-	g_hash_table_foreach_remove (priv->volume_mapping, 
+	g_hash_table_foreach_remove (priv->volume_mapping,
 				     (GHRFunc)remove_volume_by_source,
 				     source);
 	priv->sources = g_list_remove (priv->sources, source);
@@ -622,7 +383,7 @@ rb_removable_media_manager_source_deleted_cb (RBSource *source, RBRemovableMedia
 
 static void
 rb_removable_media_manager_volume_unmounted_cb (GnomeVFSVolumeMonitor *monitor,
-			     GnomeVFSVolume *volume, 
+			     GnomeVFSVolume *volume,
 			     gpointer data)
 {
 	RBRemovableMediaManager *mgr = RB_REMOVABLE_MEDIA_MANAGER (data);
@@ -640,7 +401,7 @@ rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSV
 	GnomeVFSDeviceType device_type;
 
 	g_assert (volume != NULL);
-	
+
 	if (g_hash_table_lookup (priv->volume_mapping, volume) != NULL)
 		return;
 
@@ -668,7 +429,6 @@ rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSV
 	rb_debug ("detecting new media - hal udi=%s", hal_udi);
 	rb_debug ("detecting new media - icon=%s", icon_name);
 
-	
 	/* rb_xxx_source_new first checks if the 'volume' parameter corresponds
 	 * to a medium of type 'xxx', and returns NULL if it doesn't.
 	 * When volume is of the appropriate type, it creates a new source
@@ -678,9 +438,6 @@ rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSV
 	g_signal_emit (G_OBJECT (mgr), rb_removable_media_manager_signals[CREATE_SOURCE], 0,
 		       volume, &source);
 
-	if (source == NULL && rb_audiocd_is_volume_audiocd (volume))
-		source = rb_audiocd_source_new (priv->shell, volume);
-
 	if (source) {
 		g_hash_table_insert (priv->volume_mapping, volume, source);
 		rb_removable_media_manager_append_media_source (mgr, source);
@@ -710,7 +467,6 @@ rb_removable_media_manager_unmount_volume (RBRemovableMediaManager *mgr, GnomeVF
 	}
 }
 
-
 static void
 rb_removable_media_manager_append_media_source (RBRemovableMediaManager *mgr, RBRemovableMediaSource *source)
 {
@@ -725,7 +481,7 @@ rb_removable_media_manager_append_media_source (RBRemovableMediaManager *mgr, RB
 }
 
 static void
-rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr, 
+rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 					  GtkUIManager *uimanager)
 {
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
@@ -741,6 +497,10 @@ rb_removable_media_manager_set_uimanager (RBRemovableMediaManager *mgr,
 
 	priv->uimanager = uimanager;
 
+	if (priv->uimanager != NULL) {
+		g_object_ref (priv->uimanager);
+	}
+
 	if (priv->actiongroup == NULL) {
 		priv->actiongroup = gtk_action_group_new ("RemovableMediaActions");
 		gtk_action_group_set_translation_domain (priv->actiongroup,
@@ -784,7 +544,7 @@ rb_removable_media_manager_cmd_eject_medium (GtkAction *action, RBRemovableMedia
 	RBRemovableMediaSource *source = RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source);
 	GnomeVFSVolume *volume;
 
-	g_object_get (G_OBJECT (source), "volume", &volume, NULL);
+	g_object_get (source, "volume", &volume, NULL);
 	rb_removable_media_manager_unmount_volume (mgr, volume);
 	gnome_vfs_volume_eject (volume, (GnomeVFSVolumeOpCallback)rb_removable_media_manager_eject_medium_cb, mgr);
 	gnome_vfs_volume_unref (volume);
@@ -819,16 +579,6 @@ rb_removable_media_manager_unmount_volume_swap (GnomeVFSVolume *volume, RBRemova
 	rb_removable_media_manager_unmount_volume (manager, volume);
 }
 
-#if !NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-static const char *
-nautilus_burn_drive_get_device (NautilusBurnDrive *drive)
-{
-	g_return_val_if_fail (drive != NULL, NULL);
-
-	return drive->device;
-}
-#endif
-
 void
 rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 {
@@ -839,9 +589,9 @@ rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 	struct VolumeCheckData check_data;
 
 	priv->scanned = TRUE;
-		
+
 	list = gnome_vfs_volume_monitor_get_mounted_volumes (monitor);
-	
+
 	/* see if any removable media has gone */
 	check_data.volume_list = list;
 	check_data.manager = manager;
@@ -853,7 +603,7 @@ rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 			(GFunc) rb_removable_media_manager_unmount_volume_swap,
 			manager);
 	g_list_free (check_data.volumes_to_remove);
-	
+
 	/* look for new volume media */
 	for (it = list; it != NULL; it = g_list_next (it)) {
 		volume = GNOME_VFS_VOLUME (it->data);
@@ -861,26 +611,8 @@ rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 		gnome_vfs_volume_unref (volume);
 	}
 	g_list_free (list);
-
-	/* scan cd drives */
-#if NAUTILUS_BURN_CHECK_VERSION(2,15,3)
-	list = nautilus_burn_drive_monitor_get_drives (nautilus_burn_get_drive_monitor ());
-#else
-	list = nautilus_burn_drive_get_list (FALSE, FALSE);
-#endif
-	for  (it = list; it != NULL; it = g_list_next (it)) {
-		NautilusBurnDrive *drive = (NautilusBurnDrive *)it->data;
-
-		volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, nautilus_burn_drive_get_device (drive));
-		if (volume) {
-			rb_removable_media_manager_mount_volume (manager, volume);
-			gnome_vfs_volume_unref (volume);
-		}
-	}
-	g_list_free (list);
 }
 
-
 #ifdef ENABLE_TRACK_TRANSFER
 /* Track transfer */
 
@@ -888,7 +620,7 @@ typedef struct {
 	RBRemovableMediaManager *manager;
 	RhythmDBEntry *entry;
 	char *dest;
-	char *mime_type;
+	GList *mime_types;
 	gboolean failed;
 	RBTranferCompleteCallback callback;
 	gpointer userdata;
@@ -941,7 +673,7 @@ completed_cb (RBEncoder *encoder, TransferData *data)
 
 	g_object_unref (G_OBJECT (encoder));
 	g_free (data->dest);
-	g_free (data->mime_type);
+	rb_list_deep_free (data->mime_types);
 	g_free (data);
 }
 
@@ -953,7 +685,7 @@ do_transfer (RBRemovableMediaManager *manager)
 	RBEncoder *encoder;
 
 	g_assert (rb_is_main_thread ());
-	
+
 	emit_progress (manager);
 
 	if (priv->transfer_running)
@@ -980,14 +712,14 @@ do_transfer (RBRemovableMediaManager *manager)
 	g_signal_connect (G_OBJECT (encoder),
 			  "completed", G_CALLBACK (completed_cb),
 			  data);
-	rb_encoder_encode (encoder, data->entry, data->dest, NULL);
+	rb_encoder_encode (encoder, data->entry, data->dest, data->mime_types);
 }
 
 void
 rb_removable_media_manager_queue_transfer (RBRemovableMediaManager *manager,
 					  RhythmDBEntry *entry,
 					  const char *dest,
-					  const char *mime_type,
+					  GList *mime_types,
 					  RBTranferCompleteCallback callback,
 					  gpointer userdata)
 {
@@ -1000,7 +732,7 @@ rb_removable_media_manager_queue_transfer (RBRemovableMediaManager *manager,
 	data->manager = manager;
 	data->entry = entry;
 	data->dest = g_strdup (dest);
-	data->mime_type = g_strdup (mime_type);
+	data->mime_types = rb_string_list_copy (mime_types);
 	data->callback = callback;
 	data->userdata = userdata;
 
@@ -1033,15 +765,14 @@ rb_removable_media_manager_cmd_copy_tracks (GtkAction *action, RBRemovableMediaM
 	GList *list = NULL;
 
 	source = RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source);
-	g_object_get (G_OBJECT (source), "query-model", &model, NULL);
-	g_object_get (G_OBJECT (priv->shell), "library-source", &library, NULL);
+	g_object_get (source, "query-model", &model, NULL);
+	g_object_get (priv->shell, "library-source", &library, NULL);
 
 	gtk_tree_model_foreach (GTK_TREE_MODEL (model), (GtkTreeModelForeachFunc)copy_entry, &list);
 	rb_source_paste (RB_SOURCE (library), list);
 	g_list_free (list);
 
-	g_object_unref (G_OBJECT (model));
-	g_object_unref (G_OBJECT (library));
+	g_object_unref (model);
+	g_object_unref (library);
 #endif
 }
-
