@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -37,21 +37,17 @@ static GObject *rb_import_errors_source_constructor (GType type, guint n_constru
 static void rb_import_errors_source_dispose (GObject *object);
 
 static RBEntryView *impl_get_entry_view (RBSource *source);
-static void impl_move_to_trash (RBSource *source);
 static void impl_delete (RBSource *source);
-static char *impl_get_status (RBSource *source);
+static void impl_get_status (RBSource *source, char **text, char **progress_text, float *progress);
 
 static void rb_import_errors_source_songs_show_popup_cb (RBEntryView *view,
 							 gboolean over_entry,
 							 RBImportErrorsSource *source);
 
-
 struct RBImportErrorsSourcePrivate
 {
 	RhythmDB *db;
-	RhythmDBQueryModel *model;
 	RBEntryView *view;
-	RhythmDBEntryType entry_type;
 };
 
 G_DEFINE_TYPE (RBImportErrorsSource, rb_import_errors_source, RB_TYPE_SOURCE);
@@ -77,14 +73,12 @@ rb_import_errors_source_class_init (RBImportErrorsSourceClass *klass)
 	source_class->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_false_function;
 
 	source_class->impl_delete = impl_delete;
-	source_class->impl_move_to_trash = impl_move_to_trash;
 
 	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_false_function;
 
-	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_get_status = impl_get_status;
-	
+
 	g_type_class_add_private (klass, sizeof (RBImportErrorsSourcePrivate));
 }
 
@@ -95,7 +89,7 @@ rb_import_errors_source_init (RBImportErrorsSource *source)
 	GdkPixbuf *pixbuf;
 
 	source->priv = G_TYPE_INSTANCE_GET_PRIVATE (source, RB_TYPE_IMPORT_ERRORS_SOURCE, RBImportErrorsSourcePrivate);
-	
+
 	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
 	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 					   GTK_STOCK_DIALOG_ERROR,
@@ -116,52 +110,53 @@ rb_import_errors_source_constructor (GType type, guint n_construct_properties,
 	RBImportErrorsSourceClass *klass;
 	RBShell *shell;
 	GPtrArray *query;
+	RhythmDBQueryModel *model;
+	RhythmDBEntryType entry_type;
 
 	klass = RB_IMPORT_ERRORS_SOURCE_CLASS (g_type_class_peek (RB_TYPE_IMPORT_ERRORS_SOURCE));
 
 	source = RB_IMPORT_ERRORS_SOURCE (G_OBJECT_CLASS (rb_import_errors_source_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
 
-	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
-	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	g_object_get (source,
+		      "shell", &shell,
+		      "entry-type", &entry_type,
+		      NULL);
+	g_object_get (shell, "db", &source->priv->db, NULL);
 	shell_player = rb_shell_get_player (shell);
-	g_object_unref (G_OBJECT (shell));
-	
+	g_object_unref (shell);
+
 	/* construct real query */
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
 				      	RHYTHMDB_PROP_TYPE,
-					RHYTHMDB_ENTRY_TYPE_IMPORT_ERROR,
+					entry_type,
 				      RHYTHMDB_QUERY_END);
-	source->priv->model = rhythmdb_query_model_new (source->priv->db, query,
-							(GCompareDataFunc) rhythmdb_query_model_string_sort_func,
-							RHYTHMDB_PROP_LOCATION, FALSE);
-	_rb_source_hide_when_empty (RB_SOURCE (source), source->priv->model);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
-	g_ptr_array_free (query, TRUE);
+	model = rhythmdb_query_model_new (source->priv->db, query,
+					  (GCompareDataFunc) rhythmdb_query_model_string_sort_func,
+					  GUINT_TO_POINTER (RHYTHMDB_PROP_LOCATION), NULL, FALSE);
+	rhythmdb_query_free (query);
 
 	/* set up entry view */
-	source->priv->view = rb_entry_view_new (source->priv->db, shell_player, 
+	source->priv->view = rb_entry_view_new (source->priv->db, shell_player,
 						NULL, FALSE, FALSE);
 
-	rb_entry_view_set_model (source->priv->view, source->priv->model);
+	rb_entry_view_set_model (source->priv->view, model);
 
 	rb_entry_view_append_column (source->priv->view, RB_ENTRY_VIEW_COL_LOCATION, TRUE);
 	rb_entry_view_append_column (source->priv->view, RB_ENTRY_VIEW_COL_ERROR, TRUE);
-	
-	g_signal_connect_object (G_OBJECT (source->priv->view), "show_popup",
+
+	g_signal_connect_object (source->priv->view, "show_popup",
 				 G_CALLBACK (rb_import_errors_source_songs_show_popup_cb), source, 0);
 
 	gtk_container_add (GTK_CONTAINER (source), GTK_WIDGET (source->priv->view));
-		
+
 	gtk_widget_show_all (GTK_WIDGET (source));
 
-	/* use a fake model for the source's query model property, so we can't try to play
-	 * any of the hidden entries.
-	 */
-	g_object_set (G_OBJECT (source), 
-		      "query-model", rhythmdb_query_model_new_empty (source->priv->db), 
-		      NULL);
+	g_object_set (source, "query-model", model, NULL);
+	g_object_unref (model);
 
 	return G_OBJECT (source);
 }
@@ -175,15 +170,10 @@ rb_import_errors_source_dispose (GObject *object)
 		g_object_unref (G_OBJECT (source->priv->db));
 		source->priv->db = NULL;
 	}
-	if (source->priv->model) {
-		g_object_unref (G_OBJECT (source->priv->model));
-		source->priv->model = NULL;
-	}
 
 	G_OBJECT_CLASS (rb_import_errors_source_parent_class)->dispose (object);
 }
 
-
 static RBEntryView *
 impl_get_entry_view (RBSource *asource)
 {
@@ -196,13 +186,13 @@ rb_import_errors_source_new (RBShell *shell,
 			     RBLibrarySource *library)
 {
 	RBSource *source;
-	RhythmDBEntryType entry_type;
 
-	g_object_get (G_OBJECT (library), "entry-type", &entry_type, NULL);
 	source = RB_SOURCE (g_object_new (RB_TYPE_IMPORT_ERRORS_SOURCE,
 					  "name", _("Import Errors"),
 					  "shell", shell,
 					  "visibility", FALSE,
+					  "hidden-when-empty", TRUE,
+					  "entry-type", RHYTHMDB_ENTRY_TYPE_IMPORT_ERROR,
 					  NULL));
 	return source;
 }
@@ -218,35 +208,23 @@ impl_delete (RBSource *asource)
 		rhythmdb_entry_delete (source->priv->db, tem->data);
 		rhythmdb_commit (source->priv->db);
 	}
-	g_list_free (sel);
-}
 
-static char *
-impl_get_status (RBSource *asource)
-{
-	RBImportErrorsSource *source = RB_IMPORT_ERRORS_SOURCE (asource);
-	gint count;
-	char *ret;
-	
-	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (source->priv->model), NULL);
-	ret = g_strdup_printf (ngettext ("%d import errors", "%d import errors", count),
-			       count);
-	return ret;
+	g_list_foreach (sel, (GFunc)rhythmdb_entry_unref, NULL);
+	g_list_free (sel);
 }
 
 static void
-impl_move_to_trash (RBSource *asource)
+impl_get_status (RBSource *asource, char **text, char **progress_text, float *progress)
 {
-	RBImportErrorsSource *source = RB_IMPORT_ERRORS_SOURCE (asource);
-	GList *sel, *tem;
+	RhythmDBQueryModel *model;
+	gint count;
 
-	sel = rb_entry_view_get_selected_entries (source->priv->view);
-	for (tem = sel; tem != NULL; tem = tem->next) {
-		rhythmdb_entry_move_to_trash (source->priv->db,
-					       (RhythmDBEntry *) tem->data);
-		rhythmdb_commit (source->priv->db);
-	}
-	g_list_free (sel);
+	g_object_get (asource, "query-model", &model, NULL);
+	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);
+	g_object_unref (model);
+
+	*text = g_strdup_printf (ngettext ("%d import errors", "%d import errors", count),
+				 count);
 }
 
 static void
@@ -256,4 +234,3 @@ rb_import_errors_source_songs_show_popup_cb (RBEntryView *view,
 {
 	_rb_source_show_popup (RB_SOURCE (source), "/ImportErrorsViewPopup");
 }
-
