@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of widget to display RhythmDB properties
  *
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
@@ -15,49 +16,47 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
-#include <gtk/gtktreeview.h>
+#include "config.h"
 
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtktooltips.h>
-#include <gdk/gdkkeysyms.h>
-#include <config.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <string.h>
 #include <stdlib.h>
 
+#include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+
 #include "rb-property-view.h"
 #include "rb-dialog.h"
 #include "rb-debug.h"
 #include "rhythmdb.h"
 #include "rhythmdb-property-model.h"
-#include "rb-string-helpers.h"
 #include "rb-stock-icons.h"
 #include "eel-gconf-extensions.h"
+#include "rb-util.h"
 
 static void rb_property_view_class_init (RBPropertyViewClass *klass);
 static void rb_property_view_init (RBPropertyView *view);
+static void rb_property_view_dispose (GObject *object);
 static void rb_property_view_finalize (GObject *object);
 static void rb_property_view_set_property (GObject *object,
-				       guint prop_id,
-				       const GValue *value,
-				       GParamSpec *pspec);
+					   guint prop_id,
+					   const GValue *value,
+					   GParamSpec *pspec);
 static void rb_property_view_get_property (GObject *object,
-				       guint prop_id,
-				       GValue *value,
-				       GParamSpec *pspec);
+					   guint prop_id,
+					   GValue *value,
+					   GParamSpec *pspec);
 static GObject * rb_property_view_constructor (GType type, guint n_construct_properties,
 					       GObjectConstructParam *construct_properties);
 static void rb_property_view_row_activated_cb (GtkTreeView *treeview,
-			                   GtkTreePath *path,
-			                   GtkTreeViewColumn *column,
-			                   RBPropertyView *view);
+					       GtkTreePath *path,
+					       GtkTreeViewColumn *column,
+					       RBPropertyView *view);
 static void rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 						   RBPropertyView *view);
 static void rb_property_view_pre_row_deleted_cb (RhythmDBPropertyModel *model,
@@ -65,6 +64,11 @@ static void rb_property_view_pre_row_deleted_cb (RhythmDBPropertyModel *model,
 static void rb_property_view_post_row_deleted_cb (GtkTreeModel *model,
 						  GtkTreePath *path,
 						  RBPropertyView *view);
+static gboolean rb_property_view_popup_menu_cb (GtkTreeView *treeview,
+						RBPropertyView *view);
+static gboolean rb_property_view_button_press_cb (GtkTreeView *tree,
+						  GdkEventButton *event,
+						  RBPropertyView *view);
 
 struct RBPropertyViewPrivate
 {
@@ -72,8 +76,6 @@ struct RBPropertyViewPrivate
 
 	RhythmDBPropType propid;
 
-	GPtrArray *query;
-	
 	RhythmDBPropertyModel *prop_model;
 
 	char *title;
@@ -81,17 +83,19 @@ struct RBPropertyViewPrivate
 	GtkWidget *treeview;
 	GtkTreeSelection *selection;
 
+	gboolean draggable;
 	gboolean handling_row_deletion;
-
-	guint refresh_idle_id;
 };
 
+#define RB_PROPERTY_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_PROPERTY_VIEW, RBPropertyViewPrivate))
+
 enum
 {
 	PROPERTY_SELECTED,
 	PROPERTIES_SELECTED,
 	PROPERTY_ACTIVATED,
 	SELECTION_RESET,
+	SHOW_POPUP,
 	LAST_SIGNAL
 };
 
@@ -102,51 +106,19 @@ enum
 	PROP_PROP,
 	PROP_TITLE,
 	PROP_MODEL,
+	PROP_DRAGGABLE,
 };
 
-static GObjectClass *parent_class = NULL;
-
 static guint rb_property_view_signals[LAST_SIGNAL] = { 0 };
 
-static const GtkTargetEntry target_album  [] = { { "text/x-rhythmbox-album",  0, 0 } };
-static const GtkTargetEntry target_genre  [] = { { "text/x-rhythmbox-genre",  0, 0 } };
-static const GtkTargetEntry target_artist [] = { { "text/x-rhythmbox-artist", 0, 0 } };
-
-GType
-rb_property_view_get_type (void)
-{
-	static GType rb_property_view_type = 0;
-
-	if (rb_property_view_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBPropertyViewClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_property_view_class_init,
-			NULL,
-			NULL,
-			sizeof (RBPropertyView),
-			0,
-			(GInstanceInitFunc) rb_property_view_init
-		};
-		
-		rb_property_view_type = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW,
-								"RBPropertyView",
-								&our_info, 0);
-	}
-
-	return rb_property_view_type;
-}
+G_DEFINE_TYPE (RBPropertyView, rb_property_view, GTK_TYPE_SCROLLED_WINDOW)
 
 static void
 rb_property_view_class_init (RBPropertyViewClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
+	object_class->dispose = rb_property_view_dispose;
 	object_class->finalize = rb_property_view_finalize;
 	object_class->constructor = rb_property_view_constructor;
 
@@ -166,7 +138,7 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 					 g_param_spec_enum ("prop",
 							    "PropertyId",
 							    "RhythmDBPropType",
-							    RHYTHMDB_TYPE_PROP,
+							    RHYTHMDB_TYPE_PROP_TYPE,
 							    RHYTHMDB_PROP_TYPE,
 							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
@@ -184,6 +156,13 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 							      "RhythmDBPropertyModel",
 							      RHYTHMDB_TYPE_PROPERTY_MODEL,
 							      G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_DRAGGABLE,
+					 g_param_spec_boolean ("draggable",
+						 	       "draggable",
+							       "is a drag source",
+							       TRUE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	rb_property_view_signals[PROPERTY_ACTIVATED] =
 		g_signal_new ("property-activated",
@@ -226,13 +205,89 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
-			      0); 
+			      0);
+	rb_property_view_signals[SHOW_POPUP] =
+		g_signal_new ("show_popup",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPropertyViewClass, show_popup),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
+	g_type_class_add_private (klass, sizeof (RBPropertyViewPrivate));
 }
 
 static void
 rb_property_view_init (RBPropertyView *view)
 {
-	view->priv = g_new0 (RBPropertyViewPrivate, 1);
+	view->priv = RB_PROPERTY_VIEW_GET_PRIVATE (view);
+}
+
+static void
+rb_property_view_set_model_internal (RBPropertyView *view,
+				     RhythmDBPropertyModel *model)
+{
+	if (view->priv->prop_model != NULL) {
+		g_signal_handlers_disconnect_by_func (view->priv->prop_model,
+						      G_CALLBACK (rb_property_view_pre_row_deleted_cb),
+						      view);
+		g_signal_handlers_disconnect_by_func (view->priv->prop_model,
+						      G_CALLBACK (rb_property_view_post_row_deleted_cb),
+						      view);
+		g_object_unref (view->priv->prop_model);
+	}
+
+	view->priv->prop_model = model;
+
+	if (view->priv->prop_model != NULL) {
+		GtkTreeIter iter;
+
+		g_object_ref (view->priv->prop_model);
+
+		gtk_tree_view_set_model (GTK_TREE_VIEW (view->priv->treeview),
+					 GTK_TREE_MODEL (view->priv->prop_model));
+
+		g_signal_connect_object (view->priv->prop_model,
+					 "pre-row-deletion",
+					 G_CALLBACK (rb_property_view_pre_row_deleted_cb),
+					 view,
+					 0);
+		g_signal_connect_object (view->priv->prop_model,
+					 "row_deleted",
+					 G_CALLBACK (rb_property_view_post_row_deleted_cb),
+					 view,
+					 G_CONNECT_AFTER);
+
+		g_signal_handlers_block_by_func (view->priv->selection,
+						 G_CALLBACK (rb_property_view_selection_changed_cb),
+						 view);
+
+		gtk_tree_selection_unselect_all (view->priv->selection);
+
+		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &iter))
+			gtk_tree_selection_select_iter (view->priv->selection, &iter);
+
+		g_signal_handlers_unblock_by_func (view->priv->selection,
+						   G_CALLBACK (rb_property_view_selection_changed_cb),
+						   view);
+	}
+}
+
+static void
+rb_property_view_dispose (GObject *object)
+{
+	RBPropertyView *view;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (object));
+
+	view = RB_PROPERTY_VIEW (object);
+
+	rb_property_view_set_model_internal (view, NULL);
+
+	G_OBJECT_CLASS (rb_property_view_parent_class)->dispose (object);
 }
 
 static void
@@ -245,25 +300,20 @@ rb_property_view_finalize (GObject *object)
 
 	view = RB_PROPERTY_VIEW (object);
 
-	g_return_if_fail (view->priv != NULL);
-
 	g_free (view->priv->title);
-	g_free (view->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_property_view_parent_class)->finalize (object);
 }
 
-
 static void
 rb_property_view_set_property (GObject *object,
-			   guint prop_id,
-			   const GValue *value,
-			   GParamSpec *pspec)
+			       guint prop_id,
+			       const GValue *value,
+			       GParamSpec *pspec)
 {
 	RBPropertyView *view = RB_PROPERTY_VIEW (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		view->priv->db = g_value_get_object (value);
 		break;
@@ -274,49 +324,10 @@ rb_property_view_set_property (GObject *object,
 		view->priv->title = g_value_dup_string (value);
 		break;
 	case PROP_MODEL:
-	{
-		GtkTreeIter iter;
-
-		if (view->priv->prop_model) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->prop_model),
-							      G_CALLBACK (rb_property_view_pre_row_deleted_cb),
-							      view);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->prop_model),
-							      G_CALLBACK (rb_property_view_post_row_deleted_cb),
-							      view);
-		}
-
-		view->priv->prop_model = g_value_get_object (value);
-
-		if (!view->priv->prop_model)
-			break;
-
-		gtk_tree_view_set_model (GTK_TREE_VIEW (view->priv->treeview),
-					 GTK_TREE_MODEL (view->priv->prop_model));
-
-		g_signal_connect_object (G_OBJECT (view->priv->prop_model),
-					 "pre-row-deletion",
-					 G_CALLBACK (rb_property_view_pre_row_deleted_cb),
-					 view,
-					 0);
-		g_signal_connect_object (G_OBJECT (view->priv->prop_model),
-					 "row_deleted",
-					 G_CALLBACK (rb_property_view_post_row_deleted_cb),
-					 view,
-					 G_CONNECT_AFTER);
-
-		g_signal_handlers_block_by_func (G_OBJECT (view->priv->selection),
-						 G_CALLBACK (rb_property_view_selection_changed_cb),
-						 view);
-		
-		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &iter);
-		gtk_tree_selection_unselect_all (view->priv->selection);
-		gtk_tree_selection_select_iter (view->priv->selection, &iter);
-		g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
-						   G_CALLBACK (rb_property_view_selection_changed_cb),
-						   view);
-
-	}
+		rb_property_view_set_model_internal (view, g_value_get_object (value));
+		break;
+	case PROP_DRAGGABLE:
+		view->priv->draggable = g_value_get_boolean (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -324,16 +335,15 @@ rb_property_view_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_property_view_get_property (GObject *object,
-			   guint prop_id,
-			   GValue *value,
-			   GParamSpec *pspec)
+			       guint prop_id,
+			       GValue *value,
+			       GParamSpec *pspec)
 {
 	RBPropertyView *view = RB_PROPERTY_VIEW (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_DB:
 		g_value_set_object (value, view->priv->db);
 		break;
@@ -346,6 +356,9 @@ rb_property_view_get_property (GObject *object,
 	case PROP_MODEL:
 		g_value_set_object (value, view->priv->prop_model);
 		break;
+	case PROP_DRAGGABLE:
+		g_value_set_boolean (value, view->priv->draggable);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -353,7 +366,9 @@ rb_property_view_get_property (GObject *object,
 }
 
 RBPropertyView *
-rb_property_view_new (RhythmDB *db, guint propid, const char *title)
+rb_property_view_new (RhythmDB *db,
+		      guint propid,
+		      const char *title)
 {
 	RBPropertyView *view;
 
@@ -366,6 +381,7 @@ rb_property_view_new (RhythmDB *db, guint propid, const char *title)
 					       "db", db,
 					       "prop", propid,
 					       "title", title,
+					       "draggable", TRUE,
 					       NULL));
 
 	g_return_val_if_fail (view->priv != NULL, NULL);
@@ -377,7 +393,9 @@ void
 rb_property_view_set_selection_mode (RBPropertyView *view,
 				     GtkSelectionMode mode)
 {
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
 	g_return_if_fail (mode == GTK_SELECTION_SINGLE || mode == GTK_SELECTION_MULTIPLE);
+
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (view->priv->treeview)),
 				     mode);
 }
@@ -386,26 +404,34 @@ void
 rb_property_view_reset (RBPropertyView *view)
 {
 	RhythmDBPropertyModel *model;
-	
+
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
 	model = rhythmdb_property_model_new (view->priv->db, view->priv->propid);
 
-	g_object_set (G_OBJECT (view), "property-model", model, NULL);
-	g_object_unref (G_OBJECT (model));
+	rb_property_view_set_model_internal (view, model);
+	g_object_unref (model);
 }
 
 RhythmDBPropertyModel *
 rb_property_view_get_model (RBPropertyView *view)
 {
 	RhythmDBPropertyModel *model;
-	
-	g_object_get (G_OBJECT (view), "property-model", &model, NULL);
+
+	g_return_val_if_fail (RB_IS_PROPERTY_VIEW (view), NULL);
+
+	model = view->priv->prop_model;
+
 	return model;
 }
 
 void
-rb_property_view_set_model (RBPropertyView *view, RhythmDBPropertyModel *model)
+rb_property_view_set_model (RBPropertyView *view,
+			    RhythmDBPropertyModel *model)
 {
-	g_object_set (G_OBJECT (view), "property-model", model, NULL);
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
+	rb_property_view_set_model_internal (view, model);
 }
 
 static void
@@ -426,66 +452,83 @@ rb_property_view_post_row_deleted_cb (GtkTreeModel *model,
 	if (gtk_tree_selection_count_selected_rows (view->priv->selection) == 0) {
 		GtkTreeIter first_iter;
 		rb_debug ("no rows selected, signalling reset");
-		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model),
-					       &first_iter);
-		g_signal_handlers_block_by_func (G_OBJECT (view->priv->selection),
-						 G_CALLBACK (rb_property_view_selection_changed_cb),
-						 view);
-		gtk_tree_selection_select_iter (view->priv->selection, &first_iter);
-		g_signal_emit (G_OBJECT (view), rb_property_view_signals[SELECTION_RESET], 0);
-		g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
-						   G_CALLBACK (rb_property_view_selection_changed_cb),
-						   view);
+		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &first_iter)) {
+			g_signal_handlers_block_by_func (G_OBJECT (view->priv->selection),
+							 G_CALLBACK (rb_property_view_selection_changed_cb),
+							 view);
+			gtk_tree_selection_select_iter (view->priv->selection, &first_iter);
+			g_signal_emit (G_OBJECT (view), rb_property_view_signals[SELECTION_RESET], 0);
+			g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
+							   G_CALLBACK (rb_property_view_selection_changed_cb),
+							   view);
+		}
 	}
 }
 
 guint
 rb_property_view_get_num_properties (RBPropertyView *view)
 {
+	g_return_val_if_fail (RB_IS_PROPERTY_VIEW (view), 0);
+
 	return gtk_tree_model_iter_n_children (GTK_TREE_MODEL (view->priv->prop_model),
 					       NULL)-1;
 }
 
 static void
-rb_property_view_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-				 GtkTreeModel *tree_model, GtkTreeIter *iter,
+rb_property_view_cell_data_func (GtkTreeViewColumn *column,
+				 GtkCellRenderer *renderer,
+				 GtkTreeModel *tree_model,
+				 GtkTreeIter *iter,
 				 RBPropertyView *view)
 {
+	char *title;
 	char *str;
-	gboolean bold;
+	guint number;
+	gboolean is_all;
+
+	gtk_tree_model_get (GTK_TREE_MODEL (tree_model), iter,
+			    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &title,
+			    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, &is_all,
+			    RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER, &number, -1);
+
+	if (is_all) {
+		int nodes;
+		const char *fmt;
+
+		nodes = gtk_tree_model_iter_n_children  (GTK_TREE_MODEL (tree_model), NULL);
+		/* Subtract one for the All node */
+		nodes--;
+
+		switch (view->priv->propid) {
+		case RHYTHMDB_PROP_ARTIST:
+			fmt = ngettext ("All %d artist (%d)", "All %d artists (%d)", nodes);
+			break;
+		case RHYTHMDB_PROP_ALBUM:
+			fmt = ngettext ("All %d album (%d)", "All %d albums (%d)", nodes);
+			break;
+		case RHYTHMDB_PROP_GENRE:
+			fmt = ngettext ("All %d genre (%d)", "All %d genres (%d)", nodes);
+			break;
+		default:
+			fmt = _("All %d (%d)");
+			break;
+		}
+
+		str = g_strdup_printf (fmt, nodes, number);
+	} else {
+		str = g_strdup_printf (_("%s (%d)"), title, number);
+	}
 
-	gtk_tree_model_get (GTK_TREE_MODEL (tree_model), iter, 0, &str,
-			    1, &bold, -1);
-	
 	g_object_set (G_OBJECT (renderer), "text", str,
-		      "weight", G_UNLIKELY (bold) ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL,
+		      "weight", G_UNLIKELY (is_all) ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL,
 		      NULL);
 	g_free (str);
-}
-
-
-static void
-rb_property_view_drag_data_get (GtkWidget *widget, GdkDragContext *context,
-				GtkSelectionData *selection_data, guint info,
-				guint time, gpointer data)
-{
-        const char *str;
-        RBPropertyView *view;
-        GtkTreeModel *model;
-        GtkTreeIter iter;
-        GList *selected_rows;
-
-        view = RB_PROPERTY_VIEW (data);
-
-        selected_rows = gtk_tree_selection_get_selected_rows (view->priv->selection, &model);
-        g_assert (gtk_tree_model_get_iter (model, &iter, selected_rows->data));
-        gtk_tree_model_get(model, &iter, 0, &str, -1);
-        gtk_selection_data_set (selection_data, selection_data->target,
-                                8, str, strlen (str));
+	g_free (title);
 }
 
 static GObject *
-rb_property_view_constructor (GType type, guint n_construct_properties,
+rb_property_view_constructor (GType type,
+			      guint n_construct_properties,
 			      GObjectConstructParam *construct_properties)
 {
 	GtkTreeViewColumn *column;
@@ -493,62 +536,48 @@ rb_property_view_constructor (GType type, guint n_construct_properties,
 
 	RBPropertyView *view;
 	RBPropertyViewClass *klass;
-	GObjectClass *parent_class; 
-	const GtkTargetEntry *target;
- 
-
 
 	klass = RB_PROPERTY_VIEW_CLASS (g_type_class_peek (RB_TYPE_PROPERTY_VIEW));
 
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	view = RB_PROPERTY_VIEW (parent_class->constructor (type, n_construct_properties,
-							    construct_properties));
+	view = RB_PROPERTY_VIEW (G_OBJECT_CLASS (rb_property_view_parent_class)->
+			constructor (type, n_construct_properties, construct_properties));
 
 	view->priv->prop_model = rhythmdb_property_model_new (view->priv->db, view->priv->propid);
 	view->priv->treeview = GTK_WIDGET (gtk_tree_view_new_with_model (GTK_TREE_MODEL (view->priv->prop_model)));
- 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view->priv->treeview), TRUE);
 
-        switch (view->priv->propid)
-	{
-	case RHYTHMDB_PROP_GENRE:
-		target = target_genre;
-		break;
-        case RHYTHMDB_PROP_ALBUM:
-                target = target_album;
-		break;
-        case RHYTHMDB_PROP_ARTIST:
-                target = target_artist;
-		break;
-	default:
-		target = NULL;
-		g_assert_not_reached ();
-	}
-
-        gtk_drag_source_set (GTK_WIDGET(view->priv->treeview),
-                             GDK_BUTTON1_MASK,
-                             target, 1,
-                             GDK_ACTION_COPY);
-
-        g_signal_connect_object (G_OBJECT (view->priv->treeview), "drag_data_get",
-				 G_CALLBACK (rb_property_view_drag_data_get), view, 0);
+	if (view->priv->draggable)
+		rhythmdb_property_model_enable_drag (view->priv->prop_model,
+						     GTK_TREE_VIEW (view->priv->treeview));
 
 	g_signal_connect_object (G_OBJECT (view->priv->treeview),
 			         "row_activated",
 			         G_CALLBACK (rb_property_view_row_activated_cb),
 			         view,
 				 0);
+
 	view->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->priv->treeview));
 	g_signal_connect_object (G_OBJECT (view->priv->selection),
 			         "changed",
 			         G_CALLBACK (rb_property_view_selection_changed_cb),
 			         view,
 				 0);
+	g_signal_connect_object (G_OBJECT (view->priv->treeview),
+				 "popup_menu",
+				 G_CALLBACK (rb_property_view_popup_menu_cb),
+				 view,
+				 0);
+
+	g_signal_connect_object (G_OBJECT (view->priv->treeview),
+			         "button_press_event",
+			         G_CALLBACK (rb_property_view_button_press_cb),
+			         view,
+				 0);
 
 	gtk_container_add (GTK_CONTAINER (view), view->priv->treeview);
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view->priv->treeview), TRUE);
 	gtk_tree_selection_set_mode (view->priv->selection, GTK_SELECTION_SINGLE);
-	
+
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
@@ -557,7 +586,6 @@ rb_property_view_constructor (GType type, guint n_construct_properties,
 						 view, NULL);
 	gtk_tree_view_column_set_title (column, view->priv->title);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
-
 	gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview),
 				     column);
 
@@ -571,35 +599,52 @@ rb_property_view_row_activated_cb (GtkTreeView *treeview,
 				   RBPropertyView *view)
 {
 	GtkTreeIter iter;
-	const char *val;
+	char *val;
 	gboolean is_all;
 
 	rb_debug ("row activated");
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->prop_model), &iter, path);
+	g_return_if_fail (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->prop_model),
+			  &iter, path));
 
-	gtk_tree_model_get (GTK_TREE_MODEL (view->priv->prop_model), &iter, 0,
-			    &val, 1, &is_all, -1);
+	gtk_tree_model_get (GTK_TREE_MODEL (view->priv->prop_model), &iter,
+			    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &val,
+			    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, &is_all, -1);
 
 	rb_debug ("emitting property activated");
 	g_signal_emit (G_OBJECT (view), rb_property_view_signals[PROPERTY_ACTIVATED], 0,
 		       is_all ? NULL : val);
+
+	g_free (val);
 }
 
 void
-rb_property_view_set_selection (RBPropertyView *view, const GList *vals)
+rb_property_view_set_selection (RBPropertyView *view,
+				const GList *vals)
 {
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
 	view->priv->handling_row_deletion = TRUE;
-	
+
 	gtk_tree_selection_unselect_all (view->priv->selection);
-	
+
 	for (; vals ; vals = vals->next) {
 		GtkTreeIter iter;
-		
+
 		if (rhythmdb_property_model_iter_from_string (view->priv->prop_model, vals->data, &iter)) {
+			GtkTreePath *path;
+
 			gtk_tree_selection_select_iter (view->priv->selection, &iter);
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->prop_model), &iter);
+			if (path != NULL) {
+				gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view->priv->treeview),
+							      path, NULL, TRUE,
+							      0.5, 0.0);
+				gtk_tree_path_free (path);
+			}
+
 		}
 	}
-		
+
 	view->priv->handling_row_deletion = FALSE;
 	rb_property_view_selection_changed_cb (view->priv->selection, view);
 }
@@ -608,8 +653,8 @@ static void
 rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 				       RBPropertyView *view)
 {
-	const char *selected_prop;
-	gboolean is_all;
+	char *selected_prop = NULL;
+	gboolean is_all = TRUE;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 
@@ -620,17 +665,19 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 	if (gtk_tree_selection_get_mode (selection) == GTK_SELECTION_MULTIPLE) {
 		GList *selected_rows, *tem;
 		GList *selected_properties = NULL;
-		
+
 		selected_rows = gtk_tree_selection_get_selected_rows (view->priv->selection, &model);
 		for (tem = selected_rows; tem; tem = tem->next) {
 			g_assert (gtk_tree_model_get_iter (model, &iter, tem->data));
-			gtk_tree_model_get (model, &iter, 0, &selected_prop, 1, &is_all, -1);
+			gtk_tree_model_get (model, &iter,
+					    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &selected_prop,
+					    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, &is_all, -1);
 			if (is_all) {
 				g_list_free (selected_properties);
 				selected_properties = NULL;
 				break;
 			}
-			selected_properties = g_list_append (selected_properties,
+			selected_properties = g_list_prepend (selected_properties,
 							     g_strdup (selected_prop));
 		}
 
@@ -642,19 +689,91 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 							 G_CALLBACK (rb_property_view_selection_changed_cb),
 							 view);
 			gtk_tree_selection_unselect_all (selection);
-			gtk_tree_model_get_iter_first (model, &iter);
-			gtk_tree_selection_select_iter (selection, &iter);
+			if (gtk_tree_model_get_iter_first (model, &iter))
+				gtk_tree_selection_select_iter (selection, &iter);
 			g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
 							   G_CALLBACK (rb_property_view_selection_changed_cb),
 							   view);
 		}
 		g_signal_emit (G_OBJECT (view), rb_property_view_signals[PROPERTIES_SELECTED], 0,
 			       selected_properties);
+		rb_list_deep_free (selected_properties);
 	} else {
 		if (gtk_tree_selection_get_selected (view->priv->selection, &model, &iter)) {
-			gtk_tree_model_get (model, &iter, 0, &selected_prop, 1, &is_all, -1);
+			gtk_tree_model_get (model, &iter,
+					    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &selected_prop,
+					    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, &is_all, -1);
 			g_signal_emit (G_OBJECT (view), rb_property_view_signals[PROPERTY_SELECTED], 0,
 				       is_all ? NULL : selected_prop);
 		}
 	}
+
+	g_free (selected_prop);
+}
+
+static gboolean
+rb_property_view_popup_menu_cb (GtkTreeView *treeview,
+				RBPropertyView *view)
+{
+	g_signal_emit (G_OBJECT (view), rb_property_view_signals[SHOW_POPUP], 0);
+	return TRUE;
+}
+
+void
+rb_property_view_append_column_custom (RBPropertyView *view,
+				       GtkTreeViewColumn *column)
+{
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
+	gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview), column);
+}
+
+static gboolean
+rb_property_view_button_press_cb (GtkTreeView *tree,
+				  GdkEventButton *event,
+				  RBPropertyView *view)
+{
+
+	if (event->button == 3) {
+		GtkTreeSelection *selection;
+		GtkTreePath *path;
+
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->priv->treeview));
+
+		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (view->priv->treeview), event->x, event->y, &path, NULL, NULL, NULL);
+		if (path == NULL) {
+			gtk_tree_selection_unselect_all (selection);
+		} else {
+			GtkTreeModel *model;
+			GtkTreeIter iter;
+			char *val;
+			GList *lst = NULL;
+
+			model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->priv->treeview));
+			if (gtk_tree_model_get_iter (model, &iter, path)) {
+				gtk_tree_model_get (model, &iter,
+						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &val, -1);
+				lst = g_list_prepend (lst, (gpointer) val);
+				rb_property_view_set_selection (view, lst);
+				g_free (val);
+			}
+		}
+		g_signal_emit (G_OBJECT (view), rb_property_view_signals[SHOW_POPUP], 0);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+void
+rb_property_view_set_search_func (RBPropertyView *view,
+				  GtkTreeViewSearchEqualFunc func,
+				  gpointer func_data,
+				  GtkDestroyNotify notify)
+{
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (view));
+
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (view->priv->treeview),
+					     func, func_data,
+					     notify);
 }
